package test.methodTimingTest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Applies method timing to any class or method annotated with the 
 * ApplyMethodTiming annotation.
 */
public aspect ApplyMethodTimingAspect {

    private static Log logger = LogFactory.getLog(ApplyMethodTimingAspect.class);
    
    pointcut test1(Object target, ApplyMethodTiming applyMethodTiming) : 
        execution(@ApplyMethodTiming * *(..))
        && @annotation(applyMethodTiming)
        && target(target);
    
    Object around(Object target, ApplyMethodTiming applyMethodTiming) : 
        test1(target, applyMethodTiming) {
        
        logger.fatal("1 - Applied to " + thisJoinPointStaticPart);
        return proceed(target, applyMethodTiming);
        
    }
    
    pointcut test2(Object target) : 
        execution(@ApplyMethodTiming * * (..))
        && target(target);
    
    Object around(Object target) : test2(target) {
        logger.fatal("2 - Applied to " + thisJoinPointStaticPart);
        return proceed(target);
    }
    
    pointcut test3(Object target, ApplyMethodTiming applyMethodTiming) : 
        execution(* *(..))
        && @annotation(applyMethodTiming)
        && target(target);
    
    Object around(Object target, ApplyMethodTiming applyMethodTiming) : 
        test3(target, applyMethodTiming) {
        
        logger.fatal("3 - Applied to " + thisJoinPointStaticPart);
        return proceed(target, applyMethodTiming);
        
    }
    
    pointcut shouldMatchAll(Object target) : 
        execution(* *(..))
        && target(target);
    
    Object around(Object target) : shouldMatchAll(target) {
        logger.fatal("A - Applied to " + thisJoinPointStaticPart);
        return proceed(target);
    }
    
    Object around(Object target) : shouldMatchAll(target) {
        if (null != target.getClass().getAnnotation(ApplyMethodTiming.class)) {
            logger.fatal("R - Applied to " + thisJoinPointStaticPart);
        }
        return proceed(target);
    }
}
